#!/bin/sh
#   Copyright (c) 2013 digitalSTROM.org, Zurich, Switzerland
#
#   Authors: Christian Hitz <christian.hitz@aizo.com>,
#            Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
#
#   This file is part of digitalSTROM Server.
#
#   uploadbackup.sh is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   uploadbackup.sh is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this file. If not, see <http://www.gnu.org/licenses/>.

. /www/pages/cgi-bin/webservice.sh
cd /www/pages/cgi-bin
. ./util.sh

NUM_UPLOAD_RETRIES=3
UPLOAD_SERVER="$WEBSERVICE_BASEURL"/dss/DSSBackup/SavedSSBackup

SCRIPTS_RUNNING=`ps | grep uploadbackup.sh | egrep -v grep | wc -l`
# sh forks during the $(check) so SCRIPTS_RUNNING is always off by one
SCRIPTS_RUNNING=$((SCRIPTS_RUNNING-1))
if [[ $SCRIPTS_RUNNING -gt 1 ]]; then
    log " *** Not starting backup script - other instances are already running"
    exit 0
fi

for JOB in `atq | cut -f 1`
do
    if [[ -n $JOB ]]; then
        at -c $JOB | grep uploadbackup.sh 2>&1 >/dev/null
        RET_CODE="$?"
        if [[ "$RET_CODE" -eq "0" ]]; then
            log "Removing queued upload (at #$JOB)"
            atrm $JOB
        fi
    fi
done

log "----- Running unattended backup: start"
BACKUP_UPLOADED=0
RETRY_COUNT=0
if [[ -n "$1" ]]; then
    RETRY_COUNT="$1"
fi

BACKUP_ARCHIVE=$(UNATTENDED_BACKUP=1 ./preparebackup.sh)
if [[ $? -eq 0 ]]; then
    BACKUP_NAME=$(echo $BACKUP_ARCHIVE | cut -d/ -f3)
    DSS_ID=$(mac2dsid -i eth0)
    log "Starting upload for id $DSS_ID"
    SERVER_RESPONSE=$(curl -s -F filedata=@$BACKUP_ARCHIVE $UPLOAD_SERVER?dssid=$DSS_ID)
    if [[ $? -eq 0 ]]; then
        log "Server responded with: $SERVER_RESPONSE";
        if [[ -n $SERVER_RESONSE ]]; then
            SERVER_RETCODE=$(echo $SERVER_RESPONSE | jq -M -r .ReturnCode)
            if [[ "$SERVER_RETCODE" -eq "0" ]]; then
                BACKUP_UPLOADED=1
            fi
        fi
    else
        log "Backup upload failed with curl error $?"
    fi
    rm -f $BACKUP_ARCHIVE
else
    log "Backup creation failed with error $?"
fi

if [[ "$BACKUP_UPLOADED" -eq 1 ]]; then
    log "Backup upload successful"
else
    RETRY_COUNT=$((RETRY_COUNT+1))
    if [[ $RETRY_COUNT -ge $NUM_UPLOAD_RETRIES ]]; then
        log "Failed upload retry count exceeded, not retrying anymore"
        log "----- Running unattended backup: end"
        exit 0
    fi

    # upload failed, rescedule in an hour
    log "Backup upload failed, will retry in one hour, retry count $RETRY_COUNT"
    LOGMSG=$(echo "/bin/sh /www/scripts/uploadbackup.sh $RETRY_COUNT" | at now +1 hour 2>&1)
    log "$LOGMSG"
fi

log "----- Running unattended backup: end"
